package sample.app;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Frame;
import java.awt.Image;
import java.text.MessageFormat;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import misc.Bundle;
import misc.Config;
import misc.Controller;
import misc.HelpManager;
import misc.ToolBarManager;
import misc.Util;

import sample.gui.JSample;
import sample.gui.JSampleMenuBar;
import sample.gui.JSampleToolBar;
import sample.model.Sample;

public class SampleController extends Controller<Sample> {

    // ========================================
    Sample sample;

    JSample jSample;

    JSampleMenuBar jSampleMenuBar;
    JSampleToolBar jSampleToolBar;

    SampleManager sampleManager;
    HelpManager helpManager;
    ToolBarManager toolBarManager;

    // ========================================
    public SampleController (Sample sample) {
				super (sample);
    }

    // ========================================
    protected void createModel (Sample sample) {
				this.sample = sample;
				// frames.addUpdateObserver (this, "Name");
    }

    // ========================================
    protected Component createGUI () {
				jSample = new JSample (sample);

				Frame frame = new Frame ();
				frame.setIconImage (getIcon ());
				JPanel contentPane = new JPanel (new BorderLayout ());
				toolBarManager = new ToolBarManager (getIcon (), contentPane);

				sampleManager = new SampleManager (sample, jSample);
				helpManager = new HelpManager (this, "Sample");

				jSampleToolBar = new JSampleToolBar (this, sampleManager, helpManager);
				toolBarManager.add (jSampleToolBar, BorderLayout.WEST);

				contentPane.add (jSample, BorderLayout.CENTER);

				return contentPane;
    }

    protected void newJFrame () {
				// jSampleDialog.setJFrame (jFrame);
    }

    // ========================================
    public String getTitle () {
				String name = sample.getName ();
				return MessageFormat.format (Bundle.getTitle ("Sample"),
																		 (name == null) ? Bundle.getAction ("Empty") : name);
    }

    public Image getIcon () {
				try {
						return Util.loadImageIcon (Config.getString ("SampleIcon",
																												 "data/images/sample.png")).getImage ();
				} catch (Exception e) {
						return null;
				}
    }

    public void updateName () {
				updateBundle ();
    }

    // ========================================
    protected JMenuBar createMenuBar () {
				return jSampleMenuBar = new JSampleMenuBar (this, sampleManager, helpManager);
    }

    // ========================================
    protected boolean tryClosingWindows () {
				Config.save ("Sample");
				if (sample.getModified ())
						switch (JOptionPane.showConfirmDialog (jFrame, Bundle.getMessage ("QuitJSample"),
																									 Bundle.getTitle ("SampleStillRunning"),
																									 JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE)) {
						case JOptionPane.YES_OPTION:
								return true;
						case JOptionPane.NO_OPTION:
						case JOptionPane.CANCEL_OPTION:
						case JOptionPane.CLOSED_OPTION:
								return false;
						}
				return true;
    }

    // ========================================
}
