package sample.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLayeredPane;
import javax.swing.JPanel;
import javax.swing.JRootPane;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;

import misc.Util;

import sample.model.Sample;

@SuppressWarnings ("serial") public class JSample extends JPanel {

    public static int space = 10;

    // ========================================
		Sample sample;

		JPanel jSrc =  new JPanel (null);
		JPanel jDst =  new JPanel (null);

    // ========================================
    public JSample (Sample sample) {
				super (new BorderLayout ());
				this.sample = sample;
				jSrc.setLayout (new BoxLayout (jSrc, BoxLayout.PAGE_AXIS));
				JPanel tmp = new JPanel ();
				tmp.add (jSrc);
				jDst.setPreferredSize (new Dimension (400, 400));
				add (getJSplitPane (JSplitPane.HORIZONTAL_SPLIT,
														getJScrollPane ("SRC", tmp),
														getJScrollPane ("DST", jDst)), BorderLayout.CENTER);
				sample.addUpdateObserver (this, "Data");
				updateData ();
   }

		// ======================================================================
		public void updateData () {
				invalidate ();
				jSrc.removeAll ();
				if (sample.getData ().size () <= 0)
						jDst.removeAll ();
				else
						for (String data : sample.getData ())
								jSrc.add (new JData (this, data, true));
				validate ();
				repaint ();
		}

		// ======================================================================
		public static JPanel getJScrollPane (String label, Component view) {
				JPanel result = new JPanel (new BorderLayout ());
				result.add (Util.newLabel (label, SwingConstants.LEFT), BorderLayout.NORTH);
				result.add (new JScrollPane (view, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED), BorderLayout.CENTER);
				return result;
		}

    // ========================================
		public static JSplitPane getJSplitPane (int newOrientation, Component newLeftComponent, Component newRightComponent) {
				JSplitPane result = new JSplitPane (newOrientation, newLeftComponent, newRightComponent);
				result.setResizeWeight (0.5);
        result.setOneTouchExpandable (true);
        result.setContinuousLayout (true);
				return result;
		}

    // ========================================
		JData selectedJData;
		Point relPos;

		public void startDragJData (JData jData, MouseEvent e) {
				JRootPane jRootPane = getRootPane ();
				Point absPos = jRootPane.getMousePosition (true);
				relPos = e.getPoint ();
				jData.setBackground (Color.lightGray);
				jData.setSize (e.getComponent ().getSize ());
				jData.setLocation (absPos.x-relPos.x, absPos.y-relPos.y);
				JLayeredPane jLayeredPane = jRootPane.getLayeredPane ();
				if (selectedJData != null)
						jLayeredPane.remove (selectedJData);
				selectedJData = jData;
				jLayeredPane.add (jData);
				jLayeredPane.setLayer (jData, JLayeredPane.DRAG_LAYER.intValue ());
				jData.setVisible (true);
		}

		public void dragJData () {
				try {
						Point absPos =  getRootPane ().getMousePosition (true);
						selectedJData.setLocation (absPos.x-relPos.x, absPos.y-relPos.y);
				} catch (Exception e) {
				}
		}

		public void stopDragJData () {
				if (selectedJData == null)
						return;
				selectedJData.setVisible (false);
				getRootPane ().getLayeredPane ().remove (selectedJData);

				invalidate ();
				Point absPos = jDst.getMousePosition (true);
				if (absPos == null)
						return;
				// XXX teste si la position est "raisonnable"

				jDst.add (selectedJData);
				selectedJData.setLocation (absPos.x-relPos.x, absPos.y-relPos.y);
				selectedJData.setVisible (true);
				selectedJData = null;
				validate ();
				repaint ();
		}
    // ========================================
}
